!MNH_LIC Copyright 1996-2021 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
!     #################
      MODULE MODD_BLANK_n
!     #################
!
!!****  *MODD_BLANK$n* -  Declarative module for MesoNH developpers namelist
!!
!!    PURPOSE
!!    -------
!!
!!      Offer dummy real, integer, logical and character variables for
!!    test and debugging purposes.
!!
!!**  METHOD
!!    ------
!!
!!      Eight dummy real, integer, logical and character*80 variables are
!!    defined and passed through the namelist read operations. None of the
!!    MesoNH routines uses any of those variables. When a developper choses
!!    to introduce temporarily a parameter to some subroutine, he has to
!!    introduce a USE MODD_BLANK statement into that subroutine. Then he
!!    can use any of the variables defined here and change them easily via
!!    the namelist input.
!!
!!    REFERENCE
!!    ---------
!!      None
!!
!!    AUTHOR
!!    ------
!!      K. Suhre   *Laboratoire d'Aerologie*
!!
!!    MODIFICATIONS
!!    -------------
!!
!!    Original 25/04/96
!!      updated     17/11/00  (P Jabouille) Use dummy array
!!      updated     26/10/21  (Q.Rodier) Use for n model (grid-nesting)
!-------------------------------------------------------------------------------
!
!*       0.   DECLARATIONS
!             ------------
!
USE MODD_PARAMETERS, ONLY : JPDUMMY, JPMODELMAX
!
IMPLICIT NONE
!
TYPE BLANK_t
!
  LOGICAL :: LDUMMY1
  LOGICAL :: LDUMMY2
  LOGICAL :: LDUMMY3
  LOGICAL :: LDUMMY4
  LOGICAL :: LDUMMY5
  LOGICAL :: LDUMMY6
  LOGICAL :: LDUMMY7
  LOGICAL :: LDUMMY8
!
  CHARACTER(len=80) :: CDUMMY1
  CHARACTER(len=80) :: CDUMMY2
  CHARACTER(len=80) :: CDUMMY3
  CHARACTER(len=80) :: CDUMMY4
  CHARACTER(len=80) :: CDUMMY5
  CHARACTER(len=80) :: CDUMMY6
  CHARACTER(len=80) :: CDUMMY7
  CHARACTER(len=80) :: CDUMMY8
!
  INTEGER :: NDUMMY1
  INTEGER :: NDUMMY2
  INTEGER :: NDUMMY3
  INTEGER :: NDUMMY4
  INTEGER :: NDUMMY5
  INTEGER :: NDUMMY6
  INTEGER :: NDUMMY7
  INTEGER :: NDUMMY8
!
  REAL :: XDUMMY1
  REAL :: XDUMMY2
  REAL :: XDUMMY3
  REAL :: XDUMMY4
  REAL :: XDUMMY5
  REAL :: XDUMMY6
  REAL :: XDUMMY7
  REAL :: XDUMMY8
!
END TYPE BLANK_t
!
TYPE(BLANK_t), DIMENSION(JPMODELMAX), TARGET, SAVE :: BLANK_MODEL
!
LOGICAL, POINTER :: LDUMMY1=>NULL()
LOGICAL, POINTER :: LDUMMY2=>NULL()
LOGICAL, POINTER :: LDUMMY3=>NULL()
LOGICAL, POINTER :: LDUMMY4=>NULL()
LOGICAL, POINTER :: LDUMMY5=>NULL()
LOGICAL, POINTER :: LDUMMY6=>NULL()
LOGICAL, POINTER :: LDUMMY7=>NULL()
LOGICAL, POINTER :: LDUMMY8=>NULL()
!
CHARACTER(len=80), POINTER :: CDUMMY1=>NULL()
CHARACTER(len=80), POINTER :: CDUMMY2=>NULL()
CHARACTER(len=80), POINTER :: CDUMMY3=>NULL()
CHARACTER(len=80), POINTER :: CDUMMY4=>NULL()
CHARACTER(len=80), POINTER :: CDUMMY5=>NULL()
CHARACTER(len=80), POINTER :: CDUMMY6=>NULL()
CHARACTER(len=80), POINTER :: CDUMMY7=>NULL()
CHARACTER(len=80), POINTER :: CDUMMY8=>NULL()
!
INTEGER, POINTER :: NDUMMY1=>NULL()
INTEGER, POINTER :: NDUMMY2=>NULL()
INTEGER, POINTER :: NDUMMY3=>NULL()
INTEGER, POINTER :: NDUMMY4=>NULL()
INTEGER, POINTER :: NDUMMY5=>NULL()
INTEGER, POINTER :: NDUMMY6=>NULL()
INTEGER, POINTER :: NDUMMY7=>NULL()
INTEGER, POINTER :: NDUMMY8=>NULL()
!
REAL, POINTER :: XDUMMY1=>NULL()
REAL, POINTER :: XDUMMY2=>NULL()
REAL, POINTER :: XDUMMY3=>NULL()
REAL, POINTER :: XDUMMY4=>NULL()
REAL, POINTER :: XDUMMY5=>NULL()
REAL, POINTER :: XDUMMY6=>NULL()
REAL, POINTER :: XDUMMY7=>NULL()
REAL, POINTER :: XDUMMY8=>NULL()
!
CONTAINS
!
SUBROUTINE BLANK_GOTO_MODEL(KFROM,KTO)
INTEGER, INTENT(IN) :: KFROM, KTO
!    
LDUMMY1=>BLANK_MODEL(KTO)%LDUMMY1
LDUMMY2=>BLANK_MODEL(KTO)%LDUMMY2
LDUMMY3=>BLANK_MODEL(KTO)%LDUMMY3
LDUMMY4=>BLANK_MODEL(KTO)%LDUMMY4
LDUMMY5=>BLANK_MODEL(KTO)%LDUMMY5
LDUMMY6=>BLANK_MODEL(KTO)%LDUMMY6
LDUMMY7=>BLANK_MODEL(KTO)%LDUMMY7
LDUMMY8=>BLANK_MODEL(KTO)%LDUMMY8

CDUMMY1=>BLANK_MODEL(KTO)%CDUMMY1
CDUMMY2=>BLANK_MODEL(KTO)%CDUMMY2
CDUMMY3=>BLANK_MODEL(KTO)%CDUMMY3
CDUMMY4=>BLANK_MODEL(KTO)%CDUMMY4
CDUMMY5=>BLANK_MODEL(KTO)%CDUMMY5
CDUMMY6=>BLANK_MODEL(KTO)%CDUMMY6
CDUMMY7=>BLANK_MODEL(KTO)%CDUMMY7
CDUMMY8=>BLANK_MODEL(KTO)%CDUMMY8
!
NDUMMY1=>BLANK_MODEL(KTO)%NDUMMY1
NDUMMY2=>BLANK_MODEL(KTO)%NDUMMY2
NDUMMY3=>BLANK_MODEL(KTO)%NDUMMY3
NDUMMY4=>BLANK_MODEL(KTO)%NDUMMY4
NDUMMY5=>BLANK_MODEL(KTO)%NDUMMY5
NDUMMY6=>BLANK_MODEL(KTO)%NDUMMY6
NDUMMY7=>BLANK_MODEL(KTO)%NDUMMY7
NDUMMY8=>BLANK_MODEL(KTO)%NDUMMY8
!
XDUMMY1=>BLANK_MODEL(KTO)%XDUMMY1
XDUMMY2=>BLANK_MODEL(KTO)%XDUMMY2
XDUMMY3=>BLANK_MODEL(KTO)%XDUMMY3
XDUMMY4=>BLANK_MODEL(KTO)%XDUMMY4
XDUMMY5=>BLANK_MODEL(KTO)%XDUMMY5
XDUMMY6=>BLANK_MODEL(KTO)%XDUMMY6
XDUMMY7=>BLANK_MODEL(KTO)%XDUMMY7
XDUMMY8=>BLANK_MODEL(KTO)%XDUMMY8
!
END SUBROUTINE BLANK_GOTO_MODEL
!
END MODULE MODD_BLANK_n
